{% extends 'ledger/base.html' %}
{% load i18n %}
{% load humanize %}

{% block page_title %}
    {{ title }} ⋗ {% translate "Ledger" %}
{% endblock %}

{% block page_topic %}
    <h1 class="page-header text-center">
        {{ title }}
    </h1>
{% endblock page_topic %}

{% block header_nav_collapse_left %}

{% include 'ledger/partials/menu/events/navigation.html' %}

{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}

{% include 'ledger/partials/menu/events/administration.html' %}

{% endblock header_nav_collapse_right %}

{% block ledger_block %}
<div class="container">
    <div class="row justify-content-center">
        <div class="card-body card-primary">
            <div class="card-header bg-primary">
                <h3><div class="card-title">{% translate "Delete Event" %}</div></h3>
            </div>
            <div class="card-body bg-secondary">
                <p>{% trans "Are you sure to delete the event" %} "{{ event.title }}"?</p>

                <form method="post">
                    {% csrf_token %}
                    <button type="submit" class="btn btn-danger me-2">
                        {% translate "Yes" %}
                    </button>
                    <a href="{% url 'ledger:event_admin' %}" class="btn btn-primary">
                        {% translate "Cancel" %}
                    </a>
                </form>
            </div>
        </div>
    </div>
</div>
{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_css %}
    {% include "ledger/bundles/ledger-css.html" %}
{% endblock %}

{% block extra_script %}
{% endblock %}
