{% extends 'ledger/base.html' %}
{% load i18n %}
{% load humanize %}
{% load static %}

{% block page_title %}
    {{ title }} ⋗ {% translate "Ledger" %}
{% endblock %}

{% block page_topic %}
    <h1 class="page-header text-center">
        {{ title }}
    </h1>
{% endblock page_topic %}

{% block header_nav_collapse_left %}

{% include 'ledger/partials/menu/ally/navigation.html' %}

{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}

{% include 'ledger/partials/menu/ally/administration.html' %}

{% endblock header_nav_collapse_right %}

{% block ledger_block %}
    <div class="card card-alliance-administration">
        <div class="card-header bg-primary rounded-top d-flex align-items-center">
            <h3 class="me-3">{% translate "Administration" %}</h3>
        </div>
        <!-- Content -->
        <div class="card-body">
            {% include 'ledger/partials/administration/dashboard.html' %}
        </div>
        <div class="card-body">
            <div class="card-header bg-primary rounded-top d-flex align-items-center mt-4">
                <h3 class="text-white">{% translate "Corporations" %}</h3>
            </div>
            <div class="card-body bg-secondary">
                {% include 'ledger/partials/administration/alliance.html' %}
            </div>
        </div>
    </div>
{% endblock %}

{% block extra_javascript %}
    {% include 'bundles/datatables-js-bs5.html' %}
    {% include 'ledger/bundles/ally-administration-bundles.html' %}
{% endblock %}
