{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:character_ledger_index' %}">{% translate 'Character Ledger' %}</a>
</li>

{% if perms.ledger.advanced_access %}
    <li class="nav-item">
        <a class="nav-link" href="{% url 'ledger:corporation_ledger_index' %}">{% translate 'Corporation Ledger' %}</a>
    </li>

    <li class="nav-item">
        <a class="nav-link" href="{% url 'ledger:alliance_ledger_index' %}">{% translate 'Alliance Ledger' %}</a>
    </li>
{% endif %}
